package com.android.sample;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.os.ConfigurationCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.usdk.android.UsdkThreeDS2ServiceImpl;

import org.emvco.threeds.core.AuthCallback;
import org.emvco.threeds.core.AuthSpec;
import org.emvco.threeds.core.ConfigParameters;
import org.emvco.threeds.core.DirectoryServer;
import org.emvco.threeds.core.InitCallback;
import org.emvco.threeds.core.InitSpec;
import org.emvco.threeds.core.PurchaseInfo;
import org.emvco.threeds.core.ThreeDS2Service;
import org.emvco.threeds.core.TransStatus;
import org.emvco.threeds.core.ui.UiCustomization;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_RSA_ID;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_CERT;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_EC_ID;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_EC_KEY;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_EC_KEY_ID;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_RSA_KEY;
import static com.android.sample.Constants.MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID;
import static com.android.sample.Constants.PERMISSIONS_REQUEST_CODE;
import static com.android.sample.Constants.REQUIRED_PERMISSIONS;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = MainActivity.class.getSimpleName();

    private static final String PREFS_NAME = "Msignia_Sample_App_Android";
    private static final String EMAIL_KEY = "Email";
    private static final String BACKEND_URL_KEY = "BackendUrl";

    private static ThreeDS2Service threeDS2Service;
    private static String lastTransactionId;

    private EditText emailEdit;
    private EditText cardNumberEdit;
    private EditText backendUrlEdit;
    private Button purchaseBtn;
    private Button copyLastTranIdBtn;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (threeDS2Service == null) {
            threeDS2Service = new UsdkThreeDS2ServiceImpl();

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                Log.d(TAG, "Request permissions.");
                if (!requestPermissions()) {
                    initializeThreeDsSdk();
                }
            } else {
                Log.d(TAG, "Do not request permissions.");
                initializeThreeDsSdk();
            }
        }

        TextView buildInfoTxt = findViewById(R.id.build_info_txt);
        buildInfoTxt.setText(String.format("app: %s, usdk: %s", BuildConfig.APP_VERSION, threeDS2Service.getSDKVersion()));

        emailEdit = findViewById(R.id.email_edit);
        emailEdit.setText(getEmail());

        cardNumberEdit = findViewById(R.id.card_number_edit);

        backendUrlEdit = findViewById(R.id.backend_url_edit);
        backendUrlEdit.setText(getBackendUrl());

        purchaseBtn = findViewById(R.id.purchase_btn);
        purchaseBtn.setEnabled(threeDS2Service != null);

        purchaseBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                final String email = emailEdit.getText().toString();
                if (TextUtils.isEmpty(email)) {
                    showDialog("Please, enter email", "OK");
                    return;
                }

                if (TextUtils.isEmpty(cardNumberEdit.getText().toString())) {
                    showDialog("Please, enter card number", "OK");
                    return;
                }

                String backendUrl = backendUrlEdit.getText().toString();
                if (TextUtils.isEmpty(backendUrl)) {
                    showDialog("Please, enter backend URL", "OK");
                    return;
                }

                AuthSpec authSpec = new AuthSpec();
                authSpec.setCurrentActivity(MainActivity.this);
                authSpec.setCardId(cardNumberEdit.getText().toString());
                authSpec.setAuthorizationHeaders(new HashMap<String, String>() {{
                    put("email", email);
                }});
                authSpec.setSupportedVersionsURL(backendUrl + "/api/v2/supported-versions");
                authSpec.setAuthURL(backendUrl + "/api/v2/auth");
                authSpec.setPaymentAuthentication(true);
                authSpec.setPurchaseInfo(new PurchaseInfo() {
                    @Override
                    public JSONObject toJson() {
                        JSONObject json = new JSONObject();
                        try {
                            json.put("email", email);
                            json.put("cardID", cardNumberEdit.getText().toString());
                            json.put("purchaseAmount", "2495");
                            json.put("purchaseCurrency", "840");
                            json.put("purchaseExponent", "2");
                            json.put("purchaseDate", new SimpleDateFormat("yyyyMMddHHmmss").format(Calendar.getInstance().getTime()));
                        } catch (JSONException e) {
                            Log.e(TAG, "Can not create purchase info", e);
                        }

                        return json;
                    }
                });

                threeDS2Service.authenticate(authSpec, new AuthCallback() {
                    @Override
                    public void authenticated(String transId, TransStatus transStatus) {
                        lastTransactionId = transId;
                        Log.d(TAG, "Success authentication. Transaction ID: " + transId);
                        showDialog(String.format("Success authentication. Transaction ID: %s", transId), "OK");
                    }

                    @Override
                    public void notAuthenticated(String transId, TransStatus transStatus) {
                        lastTransactionId = transId;

                        Log.d(TAG, "Unsuccessful authentication. Transaction ID: " + transId);

                        String dialogMessage;
                        if (transStatus == null) {
                            dialogMessage = String.format("Unsuccessful authentication. Transaction status is not available. Transaction ID: %s.", transId);
                        } else {
                            dialogMessage = String.format("Unsuccessful authentication. Transaction status: %s. Transaction ID: %s.", transStatus.name(), transId);
                        }
                        showDialog(dialogMessage, "OK");
                    }

                    @Override
                    public void decoupledAuthBeingPerformed(String transId, TransStatus transStatus) {
                        lastTransactionId = transId;

                        Log.d(TAG, "Decoupled authentication is being performed. Transaction ID: " + transId);

                        showDialog(String.format("Decoupled authentication is being performed. Transaction ID: %s.", transId), "OK");
                    }

                    @Override
                    public void cancelled(String transId) {
                        lastTransactionId = transId;

                        Log.d(TAG, "Authentication was cancelled. Transaction ID: " + transId);

                        showDialog(String.format("Authentication was cancelled. Transaction ID: %s.", transId), "OK");
                    }

                    @Override
                    public void error(String transId, Exception e) {
                        lastTransactionId = transId;

                        Log.e(TAG, "Failed authentication", e);

                        showDialog(String.format("Error during authentication. Transaction ID: %s.", transId), "OK");
                    }
                });
            }
        });

        copyLastTranIdBtn = findViewById(R.id.copy_last_trans_id_btn);
        copyLastTranIdBtn.setEnabled(threeDS2Service != null);

        copyLastTranIdBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (lastTransactionId != null) {
                    ClipboardManager clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
                    ClipData clip = ClipData.newPlainText("Trans ID", lastTransactionId);
                    clipboardManager.setPrimaryClip(clip);
                    Toast.makeText(getApplicationContext(), "Transaction ID is copied to Clipboard!", Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(getApplicationContext(), "There is no Transaction ID!", Toast.LENGTH_SHORT).show();
                }
            }
        });
    }

    @Override
    protected void onStop() {
        if (TextUtils.isEmpty(emailEdit.getText().toString())) {
            removeEmail();
        } else {
            saveEmail(emailEdit.getText().toString());
        }

        if (TextUtils.isEmpty(backendUrlEdit.getText().toString())) {
            removeBackendUrl();
        } else {
            saveBackendUrl(backendUrlEdit.getText().toString());
        }

        super.onStop();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        initializeThreeDsSdk();
    }

    public void initializeThreeDsSdk() {
        final ProgressDialog progress = new ProgressDialog(this);
        progress.setMessage(getString(R.string.progress_usdk_init));
        progress.setIndeterminate(true);
        progress.setCancelable(false);
        progress.show();

        InitSpec initSpec = new InitSpec();
        initSpec.setApplicationContext(this);
        initSpec.setLicenseKey(BuildConfig.LICENSE_KEY);
        initSpec.setLocale(ConfigurationCompat.getLocales(getResources().getConfiguration()).get(0).toString());
        initSpec.setUiCustomization(new UiCustomization());
        initSpec.setConfigParameters(getConfigParameters());

        threeDS2Service.initialize(initSpec, new InitCallback() {
            @Override
            public void onSuccess() {
                progress.dismiss();

                Log.d(TAG, "3DS SDK initialized successfully");
                purchaseBtn.setEnabled(true);
                copyLastTranIdBtn.setEnabled(true);
            }

            @Override
            public void onError(Exception e) {
                progress.dismiss();
                Log.e(TAG, "uSDK initialization failed");
                showDialog(e.getMessage(), "OK");
            }
        });
    }

    private ConfigParameters getConfigParameters() {
        ConfigParameters result = new ConfigParameters();

        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inPreferredConfig = Bitmap.Config.RGB_565;
        options.inSampleSize = 2;

        DirectoryServer msigniaDsRsa = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_RSA_ID, MSIGNIA_DIRECTORY_SERVER_RSA_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_RSA_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "mSIGNIA RSA");
        msigniaDsRsa.setDsLogo(BitmapFactory.decodeResource(getResources(), R.drawable.rsa_ds_logo, options));

        DirectoryServer msigniaDsEc = new DirectoryServer(MSIGNIA_DIRECTORY_SERVER_EC_ID, MSIGNIA_DIRECTORY_SERVER_EC_KEY_ID,
                MSIGNIA_DIRECTORY_SERVER_EC_KEY, MSIGNIA_DIRECTORY_SERVER_CERT, "mSIGNIA EC");
        msigniaDsEc.setDsLogo(BitmapFactory.decodeResource(getResources(), R.drawable.ec_ds_logo, options));

        result.addDirectoryServer(msigniaDsRsa);
        result.addDirectoryServer(msigniaDsEc);

        return result;
    }

    private boolean requestPermissions() {
        List<String> permissionsForRequest = new ArrayList<>();

        for (String permission : REQUIRED_PERMISSIONS) {
            if (ContextCompat.checkSelfPermission(MainActivity.this, permission) != PackageManager.PERMISSION_GRANTED) {
                permissionsForRequest.add(permission);
            }
        }

        if (!permissionsForRequest.isEmpty()) {
            ActivityCompat.requestPermissions(MainActivity.this,
                    permissionsForRequest.toArray(new String[permissionsForRequest.size()]), PERMISSIONS_REQUEST_CODE);
        }

        return !permissionsForRequest.isEmpty();
    }

    private String getEmail() {
        return getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).getString(EMAIL_KEY, getString(R.string.text_default_email));
    }

    private void saveEmail(String newEmail) {
        getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).edit().putString(EMAIL_KEY, newEmail).apply();
    }

    private void removeEmail() {
        getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).edit().remove(EMAIL_KEY).apply();
    }

    private String getBackendUrl() {
        return getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).getString(BACKEND_URL_KEY, BuildConfig.BACKEND_URL);
    }

    private void saveBackendUrl(String newBackendUrl) {
        getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).edit().putString(BACKEND_URL_KEY, newBackendUrl).apply();
    }

    private void removeBackendUrl() {
        getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE).edit().remove(BACKEND_URL_KEY).apply();
    }

    private void showDialog(String message, String buttonText) {
        AlertDialog.Builder adb = new AlertDialog.Builder(this)
                .setMessage(message);
        adb.setPositiveButton(buttonText, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {

            }
        });

        Dialog dialog = adb.create();
        dialog.setCanceledOnTouchOutside(false);
        dialog.setCancelable(false);
        dialog.show();
    }
}
